@@ -1,36 +1,11 @@
 '''
 Class for a bipartite network in long format.
 '''
-from paramsdict import ParamsDict
 import numpy as np
 import pandas as pd
 import bipartitepandas as bpd
 import warnings
 
-# Define default parameter dictionary
-plot_extended_eventstudy_params = ParamsDict({
-    'title_height': (1, 'type', (int, float),
-        '''
-            (default=1) Location of titles for subfigures.
-        ''', None),
-    'fontsize': (9, 'type', (int, float),
-        '''
-            (default=9) Font size of titles for subfigures.
-        ''', None),
-    'sharex': (True, 'type', bool,
-        '''
-            (default=True) Share x axis between subfigures.
-        ''', None),
-    'sharey': (True, 'type', bool,
-        '''
-            (default=True) Share y axis between subfigures.
-        ''', None),
-    'yticks_round': (1, 'type', int,
-        '''
-            (default=1) How many digits to round y ticks.
-        ''', None)
-})
-
 class BipartiteLong(bpd.BipartiteLongBase):
     '''
     Class for bipartite networks of firms and workers in long form. Inherits from BipartiteLongBase.
@@ -75,6 +50,16 @@ class BipartiteLong(bpd.BipartiteLongBase):
         '''
         return bpd.BipartiteEventStudy
 
+    @property
+    def _constructor_ees(self):
+        '''
+        For .to_extendedeventstudy(), tells BipartiteLongBase which event study format to use.
+
+        Returns:
+            (class): BipartiteExtendedEventStudy class
+        '''
+        return bpd.BipartiteExtendedEventStudy
+
     def get_worker_m(self, is_sorted=False):
         '''
         Get NumPy array indicating whether the worker associated with each observation is a mover.
@@ -541,246 +526,3 @@ class BipartiteLong(bpd.BipartiteLongBase):
             frame = frame.reindex(sorted_cols, axis=1, copy=False)
 
         return frame
-
-    def get_extended_eventstudy(self, transition_col='j', outcomes=None, periods_pre=3, periods_post=3, stable_pre=None, stable_post=None, is_sorted=False, copy=True):
-        '''
-        Return Pandas dataframe of event study with periods_pre periods before the transition (the transition is defined by a switch in the transition column) and periods_post periods after the transition, where transition fulcrums are given by job moves, and the first post-period is given by the job move. Returned dataframe gives worker id, period of transition, income over all periods, and firm cluster over all periods.
-
-        Arguments:
-            transition_col (str): column to use to define a transition
-            outcomes (column name or list of column names or None): columns to include data for all periods; None is equivalent to ['g', 'y']
-            periods_pre (int): number of periods before the transition
-            periods_post (int): number of periods after the transition
-            stable_pre (column name or list of column names or None): for each column, keep only workers who have constant values in that column before the transition; None is equivalent to []
-            stable_post (column name or list of column names or None): for each column, keep only workers who have constant values in that column after the transition; None is equivalent to []
-            is_sorted (bool): if False, dataframe will be sorted by i (and t, if included). Returned dataframe will be sorted. Sorting may alter original dataframe if copy is set to False. Set is_sorted to True if dataframe is already sorted.
-            copy (bool): if False, avoid copy
-
-        Returns:
-            (Pandas DataFrame): extended event study generated from long data
-        '''
-        if outcomes is None:
-            outcomes = ['g', 'y']
-
-        # Convert into lists
-        outcomes = bpd.util.to_list(outcomes)
-        if stable_pre is None:
-            stable_pre = []
-        else:
-            stable_pre = bpd.util.to_list(stable_pre)
-        if stable_post is None:
-            stable_post = []
-        else:
-            stable_post = bpd.util.to_list(stable_post)
-
-        included_cols = self._included_cols()
-        if transition_col not in included_cols:
-            raise ValueError(f'input for transition_col is {transition_col!r}, which is not included in the dataframe.')
-        for col in outcomes:
-            if col not in included_cols:
-                raise ValueError(f'input for outcomes includes general column {col!r}, which is not included in the dataframe.')
-        for col in stable_pre:
-            if col not in included_cols:
-                raise ValueError(f'input for stable_pre includes general column {col!r}, which is not included in the dataframe.')
-        for col in stable_post:
-            if col not in included_cols:
-                raise ValueError(f'input for stable_post includes general column {col!r}, which is not included in the dataframe.')
-
-        # Get list of all columns (note that stable_pre and stable_post can have columns that are not in outcomes)
-        all_cols = outcomes[:]
-        for col in set(stable_pre + stable_post):
-            if col not in all_cols:
-                all_cols.append(col)
-
-        # Create return frame
-        es_extended_frame = pd.DataFrame(self, copy=copy)
-
-        if not is_sorted:
-            # Sort data by i and t
-            es_extended_frame.sort_values(['i', 't'], inplace=True)
-
-        # Generate how many periods each worker worked
-        es_extended_frame.loc[:, 'one'] = 1
-        es_extended_frame.loc[:, 'worker_total_periods'] = es_extended_frame.groupby('i', sort=False)['one'].transform('size')
-
-        # Keep workers with enough periods (must have at least periods_pre + periods_post periods)
-        es_extended_frame = es_extended_frame.loc[es_extended_frame.loc[:, 'worker_total_periods'].to_numpy() >= periods_pre + periods_post, :]
-        es_extended_frame.reset_index(drop=True, inplace=True)
-
-        # For each worker-period, generate (how many total years - 1) they have worked at that point (e.g. if a worker started in 2005, and had data each year, then 2008 would give 3, 2009 would give 4, etc.)
-        es_extended_frame.loc[:, 'worker_periods_worked'] = es_extended_frame.groupby('i', sort=False)['one'].cumsum().to_numpy() - 1
-        es_extended_frame.drop('one', axis=1, inplace=True)
-
-        # Find periods where the worker transitioned, which can serve as fulcrums for the event study
-        i_col = es_extended_frame.loc[:, 'i'].to_numpy()
-        transition_col = es_extended_frame.loc[:, transition_col].to_numpy()
-        i_prev = bpd.util.fast_shift(i_col, 1, fill_value=-2)
-        transition_prev = np.roll(transition_col, 1)
-        moved_firms = (i_col == i_prev) & (transition_col != transition_prev)
-        es_extended_frame.loc[:, 'moved_firms'] = moved_firms
-        del i_col, transition_col, i_prev, transition_prev, moved_firms
-
-        # Compute valid moves - periods where the worker transitioned, and they also have periods_pre periods before the move, and periods_post periods after (and including) the move
-        es_extended_frame.loc[:, 'valid_move'] = \
-                                es_extended_frame.loc[:, 'moved_firms'] & \
-                                (es_extended_frame.loc[:, 'worker_periods_worked'].to_numpy() >= periods_pre) & \
-                                ((es_extended_frame.loc[:, 'worker_total_periods'].to_numpy() - es_extended_frame.loc[:, 'worker_periods_worked'].to_numpy()) >= periods_post)
-
-        # Drop irrelevant columns
-        es_extended_frame.drop(['worker_total_periods', 'worker_periods_worked', 'moved_firms'], axis=1, inplace=True)
-
-        # Only keep workers who have a valid move
-        es_extended_frame = es_extended_frame.loc[es_extended_frame.groupby('i', sort=False)['valid_move'].transform(max).to_numpy() > 0, :]
-
-        ## Compute lags and leads
-        # For column order
-        column_order = [[] for _ in range(len(outcomes))]
-        for i, col in enumerate(all_cols):
-            col_np = es_extended_frame.loc[:, col].to_numpy()
-            # Compute lagged values
-            for j in range(1, periods_pre + 1):
-                es_extended_frame.loc[:, f'{col}_l{j}'] = bpd.util.fast_shift(col_np, j, fill_value=-2)
-                if col in outcomes:
-                    column_order[i].insert(0, f'{col}_l{j}')
-            # Compute lead values
-            for j in range(periods_post): # No + 1 because base period has no shift (e.g. y becomes y_f1)
-                if j > 0:
-                    # No shift necessary for base period because already exists
-                    es_extended_frame.loc[:, f'{col}_f{j + 1}'] = bpd.util.fast_shift(col_np, -j, fill_value=-2)
-                if col in outcomes:
-                    column_order[i].append(f'{col}_f{j + 1}')
-
-        # Demarcate valid rows (all should start off True)
-        valid_rows = ~pd.isna(es_extended_frame.loc[:, col])
-        # Construct i and i_prev (these have changed from before)
-        i_col = es_extended_frame.loc[:, 'i'].to_numpy()
-        i_prev = bpd.util.fast_shift(i_col, 1, fill_value=-2)
-        # Stable pre-trend
-        for col in stable_pre:
-            col_np = es_extended_frame.loc[:, col].to_numpy()
-            for i in range(2, periods_pre + 1):
-                # Shift 1 is baseline
-                valid_rows = (valid_rows) & (np.roll(col_np, 1) == np.roll(col_np, i)) & (i_prev == bpd.util.fast_shift(i_col, i, fill_value=-3))
-
-        # Stable post-trend
-        for col in stable_post:
-            col_np = es_extended_frame.loc[:, col].to_numpy()
-            for i in range(1, periods_post):
-                # Shift 0 is baseline
-                valid_rows = (valid_rows) & (col_np == np.roll(col_np, -i)) & (i_col == bpd.util.fast_shift(i_col, -i, fill_value=-2))
-
-        # Delete i_col, i_prev
-        del i_col, i_prev
-
-        # Update with pre- and/or post-trend
-        es_extended_frame = es_extended_frame.loc[valid_rows, :]
-
-        # Rename base period to have _f1 (e.g. y becomes y_f1)
-        es_extended_frame.rename({col: col + '_f1' for col in outcomes}, axis=1, inplace=True)
-
-        # Keep rows with valid moves
-        es_extended_frame = es_extended_frame.loc[es_extended_frame.loc[:, 'valid_move'].to_numpy() == 1, :]
-        es_extended_frame.reset_index(drop=True, inplace=True)
-
-        # Drop irrelevant columns
-        es_extended_frame.drop('valid_move', axis=1, inplace=True)
-
-        # Correct datatypes
-        for i, col in enumerate(outcomes):
-            if self.col_dtype_dict[col] == 'int':
-                es_extended_frame.loc[:, column_order[i]] = es_extended_frame.loc[:, column_order[i]].astype(int, copy=False)
-
-        col_order = []
-        for order in column_order:
-            col_order += order
-
-        # Reorder columns
-        es_extended_frame = es_extended_frame.reindex(['i', 't'] + col_order, axis=1, copy=False)
-
-        # Return es_extended_frame
-        return es_extended_frame
-
-    def plot_extended_eventstudy(self, transition_col='j', outcomes=['g', 'y'], periods_pre=2, periods_post=2, stable_pre=None, stable_post=None, plot_extended_eventstudy_params=None, is_sorted=False, copy=True):
-        '''
-        Generate event study plots. If data is not clustered, will plot all transitions in a single figure.
-
-        Arguments:
-            transition_col (str): column to use to define a transition
-            outcomes (column name or list of column names or None): columns to include data for all periods; None is equivalent to ['g', 'y']
-            periods_pre (int): number of periods before the transition
-            periods_post (int): number of periods after the transition
-            stable_pre (column name or list of column names or None): for each column, keep only workers who have constant values in that column before the transition; None is equivalent to []
-            stable_post (column name or list of column names or None): for each column, keep only workers who have constant values in that column after the transition; None is equivalent to []
-            plot_extended_eventstudy_params (ParamsDict or None): dictionary of parameters for plotting. Run bpd.plot_extended_eventstudy_params().describe_all() for descriptions of all valid parameters. None is equivalent to bpd.plot_extended_eventstudy_params().
-            is_sorted (bool): if False, dataframe will be sorted by i (and t, if included). Sorting may alter original dataframe if copy is set to False. Set is_sorted to True if dataframe is already sorted.
-            copy (bool): if False, avoid copy
-        '''
-        # FIXME this method raises the following warnings:
-        # ResourceWarning: unclosed event loop <_UnixSelectorEventLoop running=False closed=False debug=False> source=self)
-        # ResourceWarning: Enable tracemalloc to get the object allocation traceback
-
-        if outcomes is None:
-            outcomes = ['g', 'y']
-
-        if plot_extended_eventstudy_params is None:
-            plot_extended_eventstudy_params = bpd.plot_extended_eventstudy_params()
-
-        from matplotlib import pyplot as plt
-
-        n_clusters = self.n_clusters()
-        added_g = False
-        if n_clusters is None:
-            # If data not clustered
-            added_g = True
-            n_clusters = 1
-            self.loc[:, 'g'] = 0
-
-        es = self.get_extended_eventstudy(transition_col=transition_col, outcomes=outcomes, periods_pre=periods_pre, periods_post=periods_post, stable_pre=stable_pre, stable_post=stable_post,is_sorted=is_sorted, copy=copy)
-
-        # Want n_clusters x n_clusters subplots
-        fig, axs = plt.subplots(nrows=n_clusters, ncols=n_clusters, sharex=plot_extended_eventstudy_params['sharex'], sharey=plot_extended_eventstudy_params['sharey'])
-        # Create lists of the x values and y columns we want
-        x_vals = []
-        y_cols = []
-        for i in range(1, periods_pre + 1):
-            x_vals.insert(0, - i)
-            y_cols.insert(0, f'y_l{i}')
-        for i in range(1, periods_post + 1):
-            x_vals.append(i)
-            y_cols.append(f'y_f{i}')
-        # Get y boundaries
-        y_min = 1000
-        y_max = -1000
-        # Generate plots
-        if n_clusters > 1:
-            for i, row in enumerate(axs):
-                for j, ax in enumerate(row):
-                    # Keep if previous firm type is i and next firm type is j
-                    es_plot = es.loc[(es.loc[:, 'g_l1'].to_numpy() == i) & (es.loc[:, 'g_f1'].to_numpy() == j), :]
-                    y = es_plot.loc[:, y_cols].mean(axis=0)
-                    yerr = es_plot.loc[:, y_cols].std(axis=0) / (len(es_plot) ** 0.5)
-                    ax.errorbar(x_vals, y, yerr=yerr, ecolor='red', elinewidth=1, zorder=2)
-                    ax.axvline(0, color='orange', zorder=1)
-                    ax.set_title(f'{i + 1} to {j + 1} (n={len(es_plot)})', y=plot_extended_eventstudy_params['title_height'], fontdict={'fontsize': plot_extended_eventstudy_params['fontsize']})
-                    ax.grid()
-                    y_min = min(y_min, ax.get_ylim()[0])
-                    y_max = max(y_max, ax.get_ylim()[1])
-        else:
-            # Plot everything
-            y = es.loc[:, y_cols].mean(axis=0)
-            yerr = es.loc[:, y_cols].std(axis=0) / (len(es) ** 0.5)
-            axs.errorbar(x_vals, y, yerr=yerr, ecolor='red', elinewidth=1, zorder=2)
-            axs.axvline(0, color='orange', zorder=1)
-            axs.set_title(f'All Transitions (n={len(es)})', y=plot_extended_eventstudy_params['title_height'], fontdict={'fontsize': plot_extended_eventstudy_params['fontsize']})
-            axs.grid()
-            y_min = min(y_min, axs.get_ylim()[0])
-            y_max = max(y_max, axs.get_ylim()[1])
-
-        if added_g:
-            # Drop g column
-            self.drop('g', axis=1, inplace=True)
-
-        # Plot
-        plt.setp(axs, xticks=np.arange(-periods_pre, periods_post + 1), yticks=np.round(np.linspace(y_min, y_max, 4), plot_extended_eventstudy_params['yticks_round']))
-        plt.tight_layout()
-        plt.show()
